//SmallTownAI V8

require("management.nut");
require("network.nut");
require("road.nut");


import("pathfinder.road", "RoadPathFinder", 4);


class SmallTownAI extends AIController {
	
    function Start();
    
}

function SmallTownAI::Start() {
	
	
	
	
	//the current expansion strategy
	local strategy = 2;
	//the minimum funding to build or expand a network
	local min_funding = 40000;
	//the minimum quality of a route to be considered for construction
	local min_route_quality = 0;
	//an array that stores all of the company's networks
	local all_networks = array(0,null);
	//our AI's current project.
	local project = null;
	//the station we are ordering buses to
	local current_station = 0;
	//the number of loops until routes are managed
	local counter = 0;
	AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_ROAD);
	
	//a list of all of our serviced towns
	::total_towns <- AITownList();
	total_towns.RemoveList(total_towns);
	
	local town_directory = AITownList();

	//set the name of our ai
	if (!AICompany.SetName("The Company")) {
		if (!AICompany.SetName("The 2nd Company")) {
			if(!AICompany.SetName("The 3rd Company")) {
				local i = 4;
				local named = false;
				while(!named) {
					if(AICompany.SetName("The " +  i.tostring() + "th Company")) {
						named = true;
						AILog.Info("Named!");
					}
					i ++;
				}      
			}
		}
	}

   //create a loop so that our ai doesn't stop
   while (true) {
		
		//choose a new strategy
		strategy = choose_strategy(town_directory, total_towns);
		
		switch(strategy) {
			case 1:
				min_funding = 30000;
				min_route_quality = 150;
			break;
			case 2:
				min_funding = 40000;
				min_route_quality = 0;
			break;
			case 3:
				min_funding = 7500;
				min_route_quality = 400;
			break;
			default:
				min_funding = 40000;
				min_route_quality = 0;
			break;
		}
		
	    //make sure we have not expanded to every town
		if(total_towns.Count() < town_directory.Count()) {
			
			//check to see if we have the money to expand
			if(check_funding() > min_funding) {
				AILog.Info("Money is present, attempting construction");
					
				if(all_networks.len() > 0) {
					//if we have a project that is not complete, expand it
					if(!all_networks.top().is_complete && all_networks.top().stations.Count() < 12 && all_networks.top() != null && all_networks.top().core_station != null) {
						AILog.Info("Planning expansion to current project");
						
						//create a list of towns
						local town_list = AITownList();
						
						//remove towns we have already serviced
						town_list.RemoveList(all_networks.top().serviced_towns);
						town_list.RemoveList(all_networks.top().failed_connections);	
						town_list.RemoveList(total_towns);
						
						//remove towns that are too far away
						town_list.Valuate(AITown.GetDistanceManhattanToTile, AITown.GetLocation(all_networks.top().network_center));
						town_list.KeepBelowValue(250);
						
						//valuate the potential of each town and remove ones below a threshhold
						town_list.Valuate(valuate_route, all_networks.top().network_center);
						town_list.KeepAboveValue(min_route_quality);
						
						//log the nuber of potential expansions found
						AILog.Info("Potential expansions " + town_list.Count());
						
						//if there are towns to expand to
						if(town_list.Count() > 0) {
							
							//choose the town with the best potential to expand to 
							town_list.Valuate(valuate_route, all_networks.top().network_center);	
							town_list.KeepTop(1);
							AILog.Info("potential rating: " + town_list.GetValue(town_list.Begin()));
							AILog.Info("Town to expand to: " + AITown.GetName(town_list.Begin()));
							
							//all_networks.top().serviced_towns.AddItem(town_list.Begin(), 0);
							local road = build_path(AIBaseStation.GetLocation(all_networks.top().core_station), AITown.GetLocation(town_list.Begin()));	//build a path to the town
							if (road == true) {
								
								//if the path is built, add the town to the network's list of serviced towns
								all_networks.top().serviced_towns.AddItem(town_list.Begin(), 0);
								
								current_station = build_station(AITown.GetLocation(town_list.Begin()));	//build a station
								if(current_station >= 0) {
									AILog.Info("Station is at tile " + current_station);
									if(all_networks.top().depot > 0) {
										all_networks.top().stations.AddItem(AIStation.GetStationID(current_station), 0);
										local bus = build_bus(all_networks.top().depot);	//build a bus
										all_networks.top().buses.AddItem(bus, 0);	//add the bus to the network's list of buses
										order_bus(bus, all_networks.top().core_station);	//order the bus to go to the first station
										order_bus_to_location(bus, current_station);	//order the bus to go to the new station
										AIVehicle.StartStopVehicle(bus);	//start the bus
										AILog.Info("bus " + bus + " Started!");
									} else {
										
										//if the depot location is lost, stop expanding the network. 
										AILog.Error("Error: Depot location lost!");
										all_networks.top().is_complete = true;
									}
								}
							} else {
								all_networks.top().failed_connections.AddItem(town_list.Begin(), 0);
							}
						} else {
							
							//add all towns connected to the network to a list of serviced towns
							total_towns.AddList(all_networks.top().serviced_towns);
							
							//show that the network is complete
							all_networks.top().is_complete = true;
							
							//start a new network
							all_networks = start_network(all_networks);
						}
					} else {
						
							//add all towns connected to the network to a list of serviced towns
							total_towns.AddList(all_networks.top().serviced_towns);
							
							//show that the network is complete
							all_networks.top().is_complete = true;
							
							//start a new network
							all_networks = start_network(all_networks);
							
					}
				} else {
					
					//sleep a random ammount of time so that two of this ai don't always choose the same town to start a network
					AIController.Sleep(AIBase.RandRange(1000));
					
					//start a new network
					all_networks = start_network(all_networks);
				}
			} else {
				if(AICompany.GetLoanAmount() < AICompany.GetMaxLoanAmount()) {
					AILog.Info("Money is lacking, taking out loan");
					take_loan(20000);
				} else {
					AILog.Info("Not enough money available, now saving");
				}
			}
			
			counter = counter + 1;
			
			if(counter >= 3) {
				//manage all of the company's vehicles
				manage_routes(all_networks);
				
				if(check_funding() >= 60000) {
					take_loan(-20000);
				}
				counter = 0;
			}
			
			this.Sleep(500);
			
		} else {
			AILog.Info("We have run out of towns to expand to!");
			while(true) {
				AIController.Sleep(100);
				manage_routes(all_networks);
				take_loan(-20000);
			}
		}
	}
}


//the function for creating a new network

function start_network(all_networks) {
	AILog.Info("Number of networks: " + all_networks.len());
	
	//create a backup in case network construction fails
	local all_networks_backup = all_networks;
	
	//add a new network to the array of networks
	all_networks.append(Network());
	
	//find a town to work in
	total_towns.RemoveItem(all_networks.top().find_first_town(total_towns));	
	
	
	//build a station
	local station_1 = build_station(AITown.GetLocation(all_networks.top().network_center));
	
	//if the station was built, continue	
	if(station_1 >= 0) {
		
		//add the station to the list of stations in the current network
		all_networks.top().stations.AddItem(AIStation.GetStationID(station_1),0);
		
		
		//build another station
		local station_2 = build_station(AITown.GetLocation(all_networks.top().network_center));	
		
		//if the second station was built
		if(station_2 >= 0) {
			all_networks.top().stations.AddItem(AIStation.GetStationID(station_2),1);
			
			//build a depot if both stations are built
			all_networks.top().depot = build_depot(AITown.GetLocation(all_networks.top().network_center));	//build a depot and associate it with the new network
			AILog.Info(all_networks.top().depot);
			
			//if the depot was built
			if(all_networks.top().depot > 0) {
				
				//set the first station to be the project core
				all_networks.top().core_station = all_networks.top().stations.Begin();
				
				local bus = build_bus(all_networks.top().depot);	//build a bus
				all_networks.top().buses.AddItem(bus, 0);	//add the bus to the network's list of buses
				order_bus(bus, all_networks.top().stations.Begin());	//order the bus to go to the first station
				order_bus(bus, all_networks.top().stations.Next());	//order the bus to go to the next station
				AIVehicle.StartStopVehicle(bus);	//start the bus
				AILog.Info("bus" + bus + " Started!");
				return all_networks;
			} else {
				
				total_towns.AddItem(all_networks.top().network_center,1);
				AILog.Error("Failed to Build First Route!");
				AIRoad.RemoveRoadStation(station_1);
				AIRoad.RemoveRoadStation(station_2);
			}
	
			
		} else {
			
			AIRoad.RemoveRoadStation(station_1);
			AIRoad.RemoveRoadStation(station_1);
			total_towns.AddItem(all_networks.top().network_center,1);
			
		}
	} else {
		total_towns.AddItem(all_networks.top().network_center,1);
	}
	return all_networks_backup;
}

