//This file contains functions related to managing money and deciding what to do

require("road.nut");


//check how much money we can spend
function check_funding () {
	return AICompany.GetBankBalance(AICompany.COMPANY_SELF);
}

//take out a loan
function take_loan (amount) {
	AICompany.SetLoanAmount(AICompany.GetLoanAmount() + amount);
}

//valuate the potential profit of a route(not actual expected profit yet)
//TODO: calculate the actual profitability of routes
function valuate_route(town1, town2) {
	
	//Get the location of both towns
	local tile1 = AITown.GetLocation(town1);
	local tile2 = AITown.GetLocation(town2);
	
	//get the distance between the two towns
	local distance = AIMap.DistanceManhattan(tile1, tile2);
	
	//get the size of both towns
	local town1size = AITown.GetPopulation(town1);
	local town2size = AITown.GetPopulation(town2);
	
	//don't give huge cities a better value than towns that are big enough
	if(town1size >= 2200) {
		town1size = 2200;
	}
	if(town2size >= 2200) {
		town2size = 2200;
	}
	
	//valuate the profit potential of the route
	local profit = ((town1size + town2size) - (40*distance)) + 3000;
	
	//return the valuation
	return profit;
}

//get the economic status of the company
function get_situation () {
	
	//variables to use to get the current situation
	local loan = AICompany.GetLoanAmount();
	local income = AICompany.GetQuarterlyIncome(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	local expenses = AICompany.GetQuarterlyExpenses(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	local company_value = AICompany.GetQuarterlyCompanyValue(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	local balance = AICompany.GetBankBalance(AICompany.COMPANY_SELF);
	local income = AICompany.GetQuarterlyIncome(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	local expenses = AICompany.GetQuarterlyExpenses(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	
	//the variable to store the situation value
	local situation = 0;
	
	//calculate the economic status of the company
	if(expenses != 0) {
		situation = (company_value - loan) + (income - expenses) + balance;
	}
	return situation;
}

//decides how aggressive to be
function choose_strategy (town_directory, total_towns) {
	
	//variables to use
	local situation = get_situation();
	local balance = AICompany.GetBankBalance(AICompany.COMPANY_SELF);
	local loan = AICompany.GetLoanAmount();
	local loan_cap = AICompany.GetMaxLoanAmount();
	local income = AICompany.GetQuarterlyIncome(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	local expenses = AICompany.GetQuarterlyExpenses(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER);
	
	// 0 is maintain, 1 is cautious expansion, 2 is normal expansion, 3 is max expansion
	local strategy = -1;
	
	//only maintain if the company has already tried to expand to every town
	if(total_towns.Count() >= town_directory.Count()) {
		strategy = 0;
		AILog.Info("Current Expansion Strategy: Maintain");
		return strategy;
		
	//expand as quickly as possible if the company is losing money
	} else if((loan_cap + balance - loan < 20000 && income < expenses) || loan_cap + balance - loan < 10000 && AICompany.GetQuarterlyIncome(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER) - AICompany.GetQuarterlyExpenses(AICompany.COMPANY_SELF, AICompany.CURRENT_QUARTER) < 2000) {
		strategy = 3;
		AILog.Info("Current Expansion Strategy: Panic!");
		return strategy;
		
	//expand cautiously if money is limited
	} else if(loan_cap + balance - loan < 50000) {
		strategy = 1;
		AILog.Info("Current Expansion Strategy: Expand cautiously");
		return strategy;
		
	//otherwise, expand normally
	} else {
		strategy = 2;
		AILog.Info("Current Expansion Strategy: Expand normally");
		return strategy;
	}
}

function manage_routes (all_networks) {
	
	//create a list of all road vehicles
	local vehicles = AIVehicleList();
	//vehicles.RemoveList(vehicles);
	
	local replacement_veh = -1;
		
	//search each complete network for unused stations
	for(local i = 0; i < (all_networks.len() - 2); i++) {
		
		AILog.Info("Reviewing Vehicles");
	
		//do this for every vehicle
		for(local veh = vehicles.Begin(); !all_networks[i].buses.IsEnd(); veh = all_networks[i].buses.Next()) {
			
			
			//replace the vehicle if it is outdated
			if(AIVehicle.GetAgeLeft(veh) < 200 && AICompany.GetBankBalance(AICompany.COMPANY_SELF ) > 10000) {
				
				AILog.Info("Replacing Vehicle " + AIVehicle.GetName(veh));
				
				if(typeof AIVehicle.GetName(veh) != "string") {
					AILog.Error("Error 1: Vehicle Null!");
				} else {
					//send the vehicle to a depot
					AIVehicle.SendVehicleToDepot(veh);
				
					//wait until the vehicle reaches the depot
					while(!AIVehicle.IsStoppedInDepot(veh)) {
						
					}
					
					//replace the old vehicle with a new one
					all_networks[i].buses.RemoveItem(veh);
					replacement_veh = build_bus(AIVehicle.GetLocation(veh));
					all_networks[i].buses.AddItem(replacement_veh, 0);
					AIOrder.CopyOrders(replacement_veh, veh);
					AIVehicle.SellVehicle(veh);
					AIVehicle.StartStopVehicle(replacement_veh);
				}
				
				
				
			}

		}
	
		//remove failed routes
		AILog.Info("Reviewing Routes");
		
		vehicles = all_networks[i].buses;
		for(local veh = vehicles.Begin(); !vehicles.IsEnd();  veh = vehicles.Next()) {
			
			if(AIVehicle.GetProfitThisYear(veh) < -50 && AIVehicle.GetProfitLastYear(veh) < -50 && AIVehicle.GetAge(veh) > 800) {
				//send the vehicle to a depot
				AIVehicle.SendVehicleToDepot(veh);
				
				AILog.Info("Removing " + AIVehicle.GetName(veh));
				
				//wait until the vehicle reaches the depot
				while(!AIVehicle.IsStoppedInDepot(veh) && veh != null) {
					
				}
				
				all_networks[i].buses.RemoveItem(veh);
				//sell the vehicle
				AIVehicle.SellVehicle(veh);
			}
		}
		
		// remove unused stations in completed routes
		AILog.Info("Reviewing Stations");
		
		//create a list of stations in the current network
		local stations = AIStationList(AIStation.STATION_BUS_STOP);
		stations.RemoveList(stations);
		
		//create a list of crowded stations in the current network
		local crowded_stations = AIStationList(AIStation.STATION_BUS_STOP);
		crowded_stations.RemoveList(crowded_stations);
		crowded_stations.AddList(all_networks[i].stations);
		crowded_stations.Valuate(AIStation.GetCargoWaiting, get_cargo_id("PASS"));
		crowded_stations.KeepAboveValue(100);
		
		//a variable for a new bus
		local new_bus = null;
			
		stations.RemoveList(stations);
		stations.AddList(all_networks[i].stations);
		
		//check each station
		for(local current_station = stations.Begin(); !stations.IsEnd(); current_station = stations.Next()) {
			
			//if a station has no passenger rating and was empty the last time this function was called, remove it
			if(AIStation.GetCargoRating(current_station, get_cargo_id("PASS")) < 1 && all_networks[i].empty_stations.HasItem(current_station)) {
				AILog.Info("Removing " + AIStation.GetName(current_station));
				all_networks[i].stations.RemoveItem(current_station);
				if(AIRoad.RemoveRoadStation(AIStation.GetLocation(current_station))) {
					AILog.Info(AIStation.GetName(current_station) + " Removed");
				}
				
			//if an empty station is not on the list of empty stations, add it
			} else if(AIStation.GetCargoRating(current_station, get_cargo_id("PASS")) < 1) {
				all_networks[i].empty_stations.AddItem(current_station,1);
				
			//if a station on the list of empty stations is not on the list of empty stations, remove it
			} else if(all_networks[i].empty_stations.HasItem(current_station)) {
				all_networks[i].empty_stations.RemoveItem(current_station);
			}
		}
		
		//add routes to overcrowded stations
			
		//only add more buses if there are less than 5 more buses than stations in the network already
		if(all_networks[i].stations.Count() > all_networks[i].buses.Count() - 5) {
			
			//add one route to each crowded station
			if(crowded_stations.Count() >= 2) {
				AILog.Info("Two or more stations in a network are overcrowded!");
				if(crowded_stations.Count() % 2 == 0) {
					for(local curr_station = crowded_stations.Begin(); !crowded_stations.IsEnd(); curr_station = crowded_stations.Next()) {
						if(new_bus == null) {
							AILog.Info("Creating a new bus");
							new_bus = build_bus(all_networks[i].depot);
							all_networks[i].buses.AddItem(new_bus, 0);
							order_bus(new_bus, curr_station);
						} else {
							order_bus(new_bus, curr_station);
							AIVehicle.StartStopVehicle(new_bus);
							new_bus = null;
						}
					}
				} else {
					crowded_stations.RemoveTop(1);
					for(local curr_station = crowded_stations.Begin(); !crowded_stations.IsEnd(); curr_station = crowded_stations.Next()) {
						if(new_bus == null) {
							new_bus = build_bus(all_networks[i].depot);
							all_networks[i].buses.AddItem(new_bus, 0);
							order_bus(new_bus, curr_station);
						} else {
							order_bus(new_bus, curr_station);
							AIVehicle.StartStopVehicle(new_bus);
							new_bus = null;
						}
					}
				}
			}
		}
	}
}
