//This file contains the network class, the class that stores information about each network


class Network {
	
	depot = null;	//the depot built for this network
	serviced_towns = null;	//the towns serviced by this network
	network_center = null;	//thw town at the center of this network
	stations = null;	//the stations in this network
	empty_stations = null;	//stations that have been empty
	buses = null;	//the vehicles in this network
	core_station = null; //the station at the center of this network
	is_complete = null; //whether the network is done expanding
	failed_connections = null; // the list of towns the company has failed to connect to this network
	
	constructor () {
		serviced_towns = AITownList();
		serviced_towns.Clear();
		failed_connections = AITownList();
		failed_connections.Clear();
		empty_stations = AIStationList(AIStation.STATION_BUS_STOP);
		empty_stations.Clear();
		stations = AIStationList(AIStation.STATION_BUS_STOP);
		buses = AIVehicleList();
		buses.Clear();
		is_complete = false;
	}
	
	function reset() {
		serviced_towns = AITownList();
		serviced_towns.Clear();
		stations = AIStationList(AIStation.STATION_BUS_STOP);
		stations.RemoveList(stations);
		buses = AIVehicleList();
		buses.Clear();
		depot = null;
		core_station = null;
		is_complete = false;
	}
	
	function find_first_town (total_towns) {
		AILog.Info("Finding a town..");
	
		//create a list of all the unserviced towns between 250 and 1100 population.
		AILog.Info("Searching for a small town...");
		
		local town_list = AITownList();
		town_list.RemoveList(total_towns);
		town_list.Valuate(AITown.GetPopulation);
		town_list.KeepAboveValue(250);
		town_list.KeepBelowValue(1100);
		
		local test_tile = -1;
		
		//remove towns with major competition
		for(local test_town = town_list.Begin(); !town_list.IsEnd(); test_town = town_list.Next()) {

			test_tile = AITown.GetLocation(test_town);
			
			//create a list of tiles near the tile being tested
			local tiles = AITileList();
			tiles.AddRectangle(test_tile - AIMap.GetTileIndex(8,8), test_tile + AIMap.GetTileIndex(8,8));

			//remove non-station tiles
			tiles.Valuate(AITile.IsStationTile);
			tiles.RemoveValue(0);
	
			// if there are still tiles, remove the tile being tested from the list of potential station locations
			if(tiles.Count() >= 2) {
				town_list.RemoveItem(test_town);
			}
			stations.Clear();
			
		}
		
		AILog.Info("Number of towns found: " + town_list.Count());
		
		//choose a random unserviced town if there are none found
		if(town_list.Count() == 0) {
			town_list = AITownList();
			town_list.RemoveList(total_towns);
			town_list.Valuate(AIBase.RandItem);
			town_list.KeepTop(1);
			network_center = town_list.Begin();
			serviced_towns.AddItem(town_list.Begin(),0);
			return town_list.Begin()
		}
		
		//choose the largest town in this list
		town_list.KeepTop(1);
		local town = town_list.Begin();
		AILog.Info("Town Found: " + AITown.GetName(town));
		
		//set the town to be the network's center
		network_center = town;
		serviced_towns.AddItem(town,0);
		AILog.Info("serviced_towns: " + serviced_towns.Count());
		
		//return the town
		return town_list.Begin();
	}
}
